# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute future reservations commands."""


from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.instances import flags as instance_flags
from googlecloudsdk.command_lib.compute.reservations import flags as reservation_flags


def GetNamePrefixFlag():
  """Gets the --name-prefix flag."""
  help_text = """\
  A name prefix for the auto-created reservations when capacity is
  delivered at the start time. Each auto-created reservation name
  starts with the name prefix.
  """
  return base.Argument('--name-prefix', help=help_text)


def GetClearNamePrefixFlag():
  """Gets the --clear-name-prefix flag."""
  help_text = """\
  Clears the name prefix for the system generated reservations.
  """
  return base.Argument(
      '--clear-name-prefix', action='store_true', help=help_text
  )


def GetTotalCountFlag(required=False):
  """Gets the --total-count flag."""
  help_text = """\
  The total number of instances for which capacity assurance is requested at a
  future time period.
  """
  return base.Argument(
      '--total-count', required=required, type=int, help=help_text
  )


def GetStartTimeFlag(required=True):
  """Gets the --start-time flag."""
  return base.Argument(
      '--start-time', required=required, type=str, help=GetStartTimeHelpText()
  )


def GetStartTimeHelpText():
  """Gets the --start-time help text."""
  help_text = """\
  Start time of the Future Reservation. The start time must be an RFC3339 valid
  string formatted by date, time, and timezone or "YYYY-MM-DDTHH:MM:SSZ"; where
  YYYY = year, MM = month, DD = day, HH = hours, MM = minutes, SS = seconds, and
  Z = timezone (i.e. 2021-11-20T07:00:00Z).
  """
  return help_text


def GetEndTimeHelpText():
  """Gets the --end-time help text."""
  help_text = """\
  End time of the Future Reservation. The end time must be an RFC3339 valid
  string formatted by date, time, and timezone or "YYYY-MM-DDTHH:MM:SSZ"; where
  YYYY = year, MM = month, DD = day, HH = hours, MM = minutes, SS = seconds, and
  Z = timezone (i.e. 2021-11-20T07:00:00Z).
  """
  return help_text


def GetAutoDeleteAutoCreatedReservationsFlag(required=False):
  """Gets the --auto-delete-auto-created-reservations flag."""
  help_text = """\
  If specified, the auto-created reservations for a future reservation
  are deleted at the end time (default) or at a specified delete time.
  """
  return base.Argument(
      '--auto-delete-auto-created-reservations',
      action=arg_parsers.StoreTrueFalseAction,
      help=help_text,
      required=required,
  )


def GetAutoCreatedReservationsDeleteTimeFlag(required=False):
  """Gets the --auto-created-reservations-delete-time flag."""
  help_text = """\
  Automatically deletes an auto-created reservations at a specific time.
  The specified time must be an RFC3339 timestamp, which must be
  formatted as "YYYY-MM-DDTHH:MM:SSZ" where YYYY = year, MM = month, DD = day,
  HH = hours, MM = minutes, SS = seconds, and Z = time zone in
  Coordinated Universal Time (UTC). For example, specify 2021-11-20T07:00:00Z.
  """
  return base.Argument(
      '--auto-created-reservations-delete-time',
      required=required,
      type=arg_parsers.Datetime.Parse,
      help=help_text,
  )


def GetAutoCreatedReservationsDurationFlag(required=False):
  """Gets the --auto-created-reservations-duration flag."""
  help_text = """\
  Automatically deletes an auto-created reservations after a specified
  number of days, hours, minutes, or seconds. For example, specify 30m
  for 30 minutes, or 1d2h3m4s for 1 day, 2 hours, 3 minutes, and 4
  seconds. For more information, see $ gcloud topic datetimes.
  """
  return base.Argument(
      '--auto-created-reservations-duration',
      required=required,
      type=arg_parsers.Duration(),
      help=help_text,
  )


def GetDurationHelpText():
  """Gets the --duration help text."""
  help_text = """\
  Alternate way of specifying time in the number of seconds to terminate
  capacity request relative to the start time of a request.
  """
  return help_text


def GetSharedSettingFlag(custom_name=None, support_folder_share_setting=False):
  """Gets the --share-setting flag."""
  help_text = """\
  Specify if this future reservation is shared, and if so, the type of sharing.
  If you omit this flag, this value is local by default.
  """
  choices = ['local', 'projects']
  if support_folder_share_setting:
    choices.append('folders')
  return base.Argument(
      custom_name if custom_name else '--share-setting',
      choices=choices,
      help=help_text,
  )


def GetShareWithFlag(custom_name=None):
  """Gets the --share-with flag."""
  help_text = """\
  If this future reservation is shared, provide a comma-separated list
  of projects that this future reservation is shared with.
  The list must contain project IDs or project numbers.
  """
  return base.Argument(
      custom_name if custom_name else '--share-with',
      type=arg_parsers.ArgList(min_length=1),
      metavar='PROJECT',
      help=help_text,
  )


def GetClearShareSettingsFlag():
  """Gets the --clear-share-settings help text."""
  help_text = """\
  Clear share settings on future reservation. This will result in non-shared
  future reservation.
  """
  return base.Argument(
      '--clear-share-settings', action='store_true', help=help_text
  )


def GetClearLocalSsdFlag():
  """Gets the --clear-local-ssd flag."""
  help_text = """\
  Remove all local ssd information on the future reservation.
  """
  return base.Argument('--clear-local-ssd', action='store_true', help=help_text)


def GetClearAcceleratorFlag():
  """Gets the --clear-accelerator flag."""
  help_text = """\
  Remove all accelerators from the future reservation.
  """
  return base.Argument(
      '--clear-accelerator', action='store_true', help=help_text
  )


def GetPlanningStatusFlag():
  """Gets the --planning-status flag."""
  help_text = """\
  The planning status of the future reservation. The default value is DRAFT.
  While in DRAFT, any changes to the future reservation's properties will be
  allowed. If set to SUBMITTED, the future reservation will submit and its
  procurementStatus will change to PENDING_APPROVAL. Once the future reservation
  is pending approval, changes to the future reservation's properties will not
  be allowed.
  """
  return base.Argument(
      '--planning-status',
      type=lambda x: x.upper(),
      choices={
          'DRAFT': 'Default planning status value.',
          'SUBMITTED': (
              'Planning status value to immediately submit the future'
              ' reservation.'
          ),
      },
      help=help_text,
  )


def GetRequireSpecificReservationFlag():
  """--require-specific-reservation flag."""
  help_text = """\
  Indicate whether the auto-created reservations can be consumed by VMs with
  "any reservation" defined. If enabled, then only VMs that target the
  auto-created reservation by name using `--reservation-affinity=specific` can
  consume from this reservation. Auto-created reservations delivered with this
  flag enabled will inherit the name of the future reservation.
  """
  return base.Argument(
      '--require-specific-reservation',
      action=arg_parsers.StoreTrueFalseAction,
      help=help_text,
  )


def GetReservationNameFlag():
  """--reservation-name flag."""
  help_text = """\
  Name of reservations where the capacity is provisioned at the time of
  delivery of  future reservations. If the reservation with the given name
  does not exist already, it is created automatically at the time of Approval
  with INACTIVE state till specified start-time. Either provide the
  reservation_name or a name_prefix.
  """
  return base.Argument(
      '--reservation-name',
      type=str,
      help=help_text,
  )


def GetDeploymentTypeFlag():
  """--deployment-type flag."""
  help_text = """\
  The deployment type for the reserved capacity.
  """
  return base.Argument(
      '--deployment-type',
      choices={
          'DENSE': 'DENSE mode is for densely deployed reservation blocks.',
          'FLEXIBLE': (
              'FLEXIBLE mode is for highly flexible, logical reservation'
              ' blocks.'
          ),
      },
      help=help_text,
  )


def GetCommitmentNameHelpText():
  help_text = """\
  Name of commitment covering the delivered reservation at the time of delivery
  of future reservations. If not specified, it takes the name of the future
  reservation.
  """
  return help_text


def GetCommitmentPlanHelpText():
  help_text = """\
  The plan for this commitment to be created, which determines duration and
  discount rate. The currently supported plans are TWELVE_MONTH (1 year), and
  THIRTY_SIX_MONTH (3 years).
  """
  return help_text


def GetPreviousCommitmentTermsHelpText():
  help_text = """\
  Applicable only if future reservation will deliver to an existing reservation
  with a ramp plan. When set to EXTEND, all associated parent Committed Used
  Discount's end-date/term will be extended to the end-time of this future
  reservation. Default is to extend previous commitment's time to the end_time
  of the reservation.
  """
  return help_text


def GetSchedulingTypeFlag():
  """--scheduling-type flag."""
  help_text = """\
  Maintenance for the reserved capacity.
  """
  return base.Argument(
      '--scheduling-type',
      choices={
          'GROUPED': (
              'In GROUPED mode, maintenance on all reserved instances is'
              ' synchronized.'
          ),
          'INDEPENDENT': (
              'In INDEPENDENT mode, maintenance is not synchronized for this'
              ' reservation, and each instance has its own maintenance window.'
          ),
      },
      help=help_text,
  )


def GetResourceMangerTagsFlag():
  """Gets the --resource-manager-tags flag."""
  help_text = """\
  Resource manager tags to be bound to the future reservation.
  """
  return base.Argument(
      '--resource-manager-tags',
      metavar='KEY=VALUE',
      type=arg_parsers.ArgDict(),
      help=help_text,
  )


def GetReservationModeFlag():
  """--reservation-mode flag."""
  help_text = """\
  The mode of the reservation.
  """
  return base.Argument(
      '--reservation-mode',
      choices={
          'CALENDAR': (
              'This indicates to create a future reservation in calendar mode,'
              ' which is ideal for reserving GPU VMs. The auto-created'
              ' reservations for the future reservation are automatically'
              ' deleted at the end of the reservation period.'
          ),
          'DEFAULT': (
              'This indicates to create a standard future reservation. If you'
              ' want to automatically delete the auto-created reservations,'
              ' then you must use the --auto-delete-auto-created-reservations'
              ' flag.'
          ),
      },
      help=help_text,
  )


def GetEnableEmergentMaintenanceFlag():
  """--emergent-maintenance flag."""
  help_text = """\
  Emergent maintenance flag for the reservation, which enrolls all the
  underlying vms, hosts and SB infrastructure to receive emergent maintenance
  notifications in advance.
  """
  return base.Argument(
      '--enable-emergent-maintenance',
      action=arg_parsers.StoreTrueFalseAction,
      help=help_text,
  )


def AddCreateFlags(
    parser,
    support_location_hint=False,
    support_share_setting=False,
    support_folder_share_setting=False,
    support_fleet=False,
    support_instance_template=False,
    support_planning_status=False,
    support_local_ssd_count=False,
    support_auto_delete=False,
    support_require_specific_reservation=False,
    support_gsc=False,
    support_cuds=False,
    support_dws_gpu=False,
    support_dws_tpu=False,
    support_resource_manager_tags=False,
):
  """Adds all flags needed for the create command."""
  GetNamePrefixFlag().AddToParser(parser)
  GetTotalCountFlag().AddToParser(parser)
  if support_require_specific_reservation:
    GetRequireSpecificReservationFlag().AddToParser(parser)
  reservation_flags.GetDescriptionFlag(is_fr=True).AddToParser(parser)
  if support_planning_status:
    GetPlanningStatusFlag().AddToParser(parser)

  reservation_properties_group = base.ArgumentGroup(
      'To create a future reservation request, specify the properties of the'
      ' resources that you want to reserve and when you want to start using'
      ' them. After the request is approved, Compute Engine automatically'
      ' creates reservations for your requested resources at your specified'
      ' start time.',
      required=True,
      mutex=True,
  )

  if support_instance_template:
    reservation_properties_group.AddArgument(
        reservation_flags.GetSourceInstanceTemplateFlag()
    )

  AddTimeWindowFlags(parser, time_window_requird=True)

  instance_properties_group = base.ArgumentGroup(
      'Define individual instance properties for the specific SKU reservation.'
  )
  instance_properties_group.AddArgument(reservation_flags.GetMachineType())
  instance_properties_group.AddArgument(reservation_flags.GetMinCpuPlatform())
  if support_local_ssd_count:
    instance_properties_group.AddArgument(
        reservation_flags.GetLocalSsdFlagWithCount()
    )
  else:
    instance_properties_group.AddArgument(reservation_flags.GetLocalSsdFlag())
  instance_properties_group.AddArgument(reservation_flags.GetAcceleratorFlag())
  if support_location_hint:
    instance_properties_group.AddArgument(reservation_flags.GetLocationHint())
  if support_fleet:
    instance_properties_group.AddArgument(
        instance_flags.AddMaintenanceFreezeDuration()
    )
    instance_properties_group.AddArgument(
        instance_flags.AddMaintenanceInterval()
    )

  if support_dws_tpu:
    aggregate_reservation_group = base.ArgumentGroup(
        'You must define the version and number of TPUs to reserve.'
    )
    aggregate_reservation_group.AddArgument(
        reservation_flags.GetTpuVersion()
    )
    aggregate_reservation_group.AddArgument(
        reservation_flags.GetChipCount()
    )
    aggregate_reservation_group.AddArgument(
        reservation_flags.GetWorkloadType()
    )
    reservation_properties_group.AddArgument(aggregate_reservation_group)

  reservation_properties_group.AddArgument(instance_properties_group)
  reservation_properties_group.AddToParser(parser)

  if support_share_setting:
    share_group = base.ArgumentGroup(
        'Manage the properties of a shared reservation.', required=False
    )
    share_group.AddArgument(
        GetSharedSettingFlag(
            support_folder_share_setting=support_folder_share_setting
        )
    )
    share_group.AddArgument(GetShareWithFlag())
    share_group.AddToParser(parser)

  if support_auto_delete:
    AddAutoDeleteFlags(parser)

  if support_gsc:
    GetReservationNameFlag().AddToParser(parser)
    GetDeploymentTypeFlag().AddToParser(parser)
    GetSchedulingTypeFlag().AddToParser(parser)

  if support_cuds:
    AddCommitmentInfoFlags(parser)

  if support_dws_gpu:
    GetReservationModeFlag().AddToParser(parser)

  if support_resource_manager_tags:
    GetResourceMangerTagsFlag().AddToParser(parser)


def AddUpdateFlags(
    parser,
    support_location_hint=False,
    support_fleet=False,
    support_planning_status=False,
    support_local_ssd_count=False,
    support_share_setting=False,
    support_auto_delete=False,
    support_require_specific_reservation=False,
    support_gsc=False,
    support_cuds=False,
    support_emergent_maintenance=False,
):
  """Adds all flags needed for the update command."""

  name_prefix_group = base.ArgumentGroup(
      'Manage the name-prefix of a future reservation.',
      required=False,
      mutex=True
  )
  name_prefix_group.AddArgument(GetNamePrefixFlag())
  name_prefix_group.AddArgument(GetClearNamePrefixFlag())
  name_prefix_group.AddToParser(parser)

  GetTotalCountFlag(required=False).AddToParser(parser)
  reservation_flags.GetDescriptionFlag(is_fr=True).AddToParser(parser)

  if support_planning_status:
    GetPlanningStatusFlag().AddToParser(parser)
  group = base.ArgumentGroup(
      'Manage the specific SKU reservation properties.', required=False
  )
  group.AddArgument(reservation_flags.GetMachineType(required=False))
  group.AddArgument(reservation_flags.GetMinCpuPlatform())

  accelerator_group = base.ArgumentGroup(
      'Manage the accelerators of a future reservation.',
      required=False,
      mutex=True,
  )
  accelerator_group.AddArgument(reservation_flags.GetAcceleratorFlag())
  accelerator_group.AddArgument(GetClearAcceleratorFlag())
  group.AddArgument(accelerator_group)

  local_ssd_group = base.ArgumentGroup(
      'Manage the local ssd of a future reservation.',
      required=False,
      mutex=True,
  )
  if support_local_ssd_count:
    local_ssd_group.AddArgument(reservation_flags.GetLocalSsdFlagWithCount())
  else:
    local_ssd_group.AddArgument(reservation_flags.GetLocalSsdFlag())
  local_ssd_group.AddArgument(GetClearLocalSsdFlag())
  group.AddArgument(local_ssd_group)

  if support_location_hint:
    group.AddArgument(reservation_flags.GetLocationHint())
  if support_fleet:
    group.AddArgument(instance_flags.AddMaintenanceInterval())
  group.AddToParser(parser)
  AddTimeWindowFlags(parser, time_window_requird=False)

  if support_share_setting:
    share_group = base.ArgumentGroup(
        'Manage the properties of a shared future reservation.',
        required=False,
        mutex=True,
    )
    share_group.AddArgument(GetClearShareSettingsFlag())

    share_setting_group = base.ArgumentGroup(
        'Manage the share settings of a future reservation.', required=False
    )
    share_setting_group.AddArgument(GetSharedSettingFlag())
    share_setting_group.AddArgument(GetShareWithFlag())

    share_group.AddArgument(share_setting_group)
    share_group.AddToParser(parser)

  if support_auto_delete:
    AddAutoDeleteFlags(parser, is_update=True)

  if support_require_specific_reservation:
    GetRequireSpecificReservationFlag().AddToParser(parser)

  if support_gsc:
    GetReservationNameFlag().AddToParser(parser)
    GetDeploymentTypeFlag().AddToParser(parser)
    GetSchedulingTypeFlag().AddToParser(parser)

  if support_cuds:
    AddCommitmentInfoFlags(parser)

  if support_emergent_maintenance:
    GetEnableEmergentMaintenanceFlag().AddToParser(parser)


def AddAutoDeleteFlags(parser, is_update=False):
  """Adds all flags needed for the modifying the auto-delete properties."""

  GetAutoDeleteAutoCreatedReservationsFlag(
      required=False if is_update else True
  ).AddToParser(parser)

  auto_delete_time_settings_group = base.ArgumentGroup(
      'Manage the auto-delete time properties.',
      required=False,
      mutex=True,
  )

  auto_delete_time_settings_group.AddArgument(
      GetAutoCreatedReservationsDeleteTimeFlag()
  )
  auto_delete_time_settings_group.AddArgument(
      GetAutoCreatedReservationsDurationFlag()
  )

  auto_delete_time_settings_group.AddToParser(parser)


def AddTimeWindowFlags(parser, time_window_requird=False):
  """Adds all flags needed for the modifying the time window properties."""

  time_window_group = parser.add_group(
      help='Manage the time specific properties for requesting future capacity',
      required=time_window_requird,
  )
  time_window_group.add_argument(
      '--start-time', required=time_window_requird, help=GetStartTimeHelpText()
  )
  end_time_window_group = time_window_group.add_mutually_exclusive_group(
      required=time_window_requird
  )
  end_time_window_group.add_argument('--end-time', help=GetEndTimeHelpText())
  end_time_window_group.add_argument(
      '--duration', type=int, help=GetDurationHelpText()
  )


def AddCommitmentInfoFlags(parser):
  """Adds all flags needed for the modifying the commitment info properties."""

  commitment_info_group = parser.add_group(
      help='Manage the commitment info properties',
      required=False,
  )
  commitment_info_group.add_argument(
      '--commitment-name', type=str, help=GetCommitmentNameHelpText()
  )
  commitment_info_group.add_argument(
      '--commitment-plan',
      choices=['TWELVE_MONTH', 'THIRTY_SIX_MONTH'],
      help=GetCommitmentPlanHelpText(),
  )
  commitment_info_group.add_argument(
      '--previous-commitment-terms',
      choices=['EXTEND'],
      help=GetPreviousCommitmentTermsHelpText(),
  )



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Describe organization rollout plan command."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
@base.UniverseCompatible
class Describe(base.DescribeCommand):
  """Describe an organization rollout plan."""

  detailed_help = {
      'brief': 'Describe an organization rollout plan.',
      'DESCRIPTION': 'Describe an organization rollout plan.',
      'EXAMPLES': (
          r"""
    To describe an organization rollout plan named 'my-org-rollout-plan' in organization '123456789', run:

      $ {command} my-org-rollout-plan --organization=123456789
    """
      ),
  }

  @staticmethod
  def Args(parser):
    org_rollout_plan_resource_spec = concepts.ResourceSpec(
        'compute.organizationRolloutPlans',
        api_version='beta',
        resource_name='organization rollout plan',
        organizationsId=concepts.ResourceParameterAttributeConfig(
            name='organization', help_text='The Google Cloud organization ID.'
        ),
        rolloutPlan=concepts.ResourceParameterAttributeConfig(
            name='name',
            help_text='Name of the organization rollout plan to describe.',
        ),
    )
    presentation_spec = presentation_specs.ResourcePresentationSpec(
        'name',
        org_rollout_plan_resource_spec,
        'Name of the organization rollout plan you want to inspect.',
        required=True,
    )
    concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    service = client.apitools_client.organizationRolloutPlans
    messages = holder.client.messages
    org_rollout_plan_ref = args.CONCEPTS.name.Parse()

    request = messages.ComputeOrganizationRolloutPlansGetRequest(
        organization='organizations/' + org_rollout_plan_ref.organizationsId,
        rolloutPlan=org_rollout_plan_ref.Name(),
    )

    return service.Get(request)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a Cloud SQL blue-green deployment."""

import argparse
from typing import Any, Dict

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.sql import api_util
from googlecloudsdk.api_lib.sql import operations
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.sql import flags as sql_flags
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io


@base.Hidden
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Create(base.CreateCommand):
  """Creates a Cloud SQL blue-green deployment."""

  @classmethod
  def Args(cls, parser: argparse.ArgumentParser) -> None:
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    del cls  # Unused.
    sql_flags.AddDeploymentId(parser)
    sql_flags.AddSourceInstance(parser)
    sql_flags.AddRegion(parser, required=False, specify_default_region=False)
    sql_flags.AddDeploymentDescription(parser)
    sql_flags.AddTargetDatabaseVersion(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    parser.display_info.AddFormat('default')

  def Run(self, args: argparse.Namespace) -> Dict[str, Any]:
    """Creates a Cloud SQL blue-green deployment.

    Args:
      args: The arguments that this command was invoked with.

    Returns:
      A dictionary object representing the long-running operation resource if
      the create was successful.
    Raises:
      HttpException: An HTTP error response was received while executing API
          request.
    """
    client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
    sql_client = client.sql_client
    sql_messages = client.sql_messages
    project = properties.VALUES.core.project.GetOrFail()
    region = args.region
    if not region:
      region = properties.VALUES.compute.region.Get()
    if not region:
      raise calliope_exceptions.RequiredArgumentException(
          '--region',
          'Region not chosen. Specify command-line flag --region or region'
          ' property.'
      )
    parent = f'projects/{project}/locations/{region}'
    source_instance_name = args.source_instance

    console_io.PromptContinue(
        message=(
            f'Blue green deployment {args.deployment} will be created in'
            f' project {project} and region {region}.'
        ),
        cancel_on_no=True,
    )

    try:
      requested_config = None
      if args.target_database_version:
        requested_config = sql_messages.RequestedConfig(
            databaseVersion=args.target_database_version
        )

      deployment_resource = sql_messages.BlueGreenDeployment(
          sourceInstance=source_instance_name,
          description=args.description,
          requestedConfig=requested_config,
      )

      request = sql_messages.SqlBlueGreenDeploymentsCreateRequest(
          parent=parent,
          blueGreenDeploymentId=args.deployment,
          blueGreenDeployment=deployment_resource,
      )

      result_operation = sql_client.blueGreenDeployments.Create(request)
      operation_ref = client.resource_parser.Create(
          'sql.operations', operation=result_operation.name, project=project
      )

      if args.async_:
        return result_operation

      operations.OperationsV1Beta4.WaitForOperation(
          sql_client, operation_ref, 'Creating blue-green deployment'
      )

      return sql_client.blueGreenDeployments.Get(
          sql_messages.SqlBlueGreenDeploymentsGetRequest(
              name=f'{parent}/blueGreenDeployments/{args.deployment}'
          )
      )

    except apitools_exceptions.HttpError as error:
      raise calliope_exceptions.HttpException(error) from error



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Performs a point in time restore for a Cloud SQL instance managed by Google Cloud Backup and Disaster Recovery (DR) Service."""


from googlecloudsdk.api_lib.sql import api_util
from googlecloudsdk.api_lib.sql import operations
from googlecloudsdk.api_lib.sql import validate
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.command_lib.sql import constants
from googlecloudsdk.command_lib.sql import flags
from googlecloudsdk.command_lib.sql import instances as command_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources
from googlecloudsdk.generated_clients.apis.sqladmin.v1beta4 import sqladmin_v1beta4_messages


DESCRIPTION = """\

    *{command}* performs a point in time restore for a Cloud SQL instance
    managed by Google Cloud Backup and Disaster Recovery (DR) Service.

    If point-in-time restore is performed using a multiregion datasource,
    the '--region' flag is required to specify the target region for
    the restore.
    NOTE: The target region must be the same as the region of the source
    instance.
    """

EXAMPLES_GA = """\
    To perform a point in time restore from an earlier point in time:

      $ {command} datasource target-instance '2012-11-15T16:19:00.094Z'

    To perform a point in time restore from an earlier point in time to a
    target region, specify the --region flag:

      $ {command} datasource target-instance '2012-11-15T16:19:00.094Z' --region=us-central1

    """

DETAILED_HELP = {
    'DESCRIPTION': DESCRIPTION,
    'EXAMPLES': EXAMPLES_GA,
}


def _GetInstanceRefFromArgs(
    args: parser_extensions.Namespace, client: api_util.SqlClient
) -> resources.Resource:
  """Get validated ref to destination instance from args."""

  validate.ValidateInstanceName(args.target)
  return client.resource_parser.Parse(
      args.target,
      params={'project': properties.VALUES.core.project.GetOrFail},
      collection='sql.instances',
  )


def _UpdateRequestFromArgs(
    request: sqladmin_v1beta4_messages.SqlInstancesPointInTimeRestoreRequest,
    args: parser_extensions.Namespace,
) -> None:
  """Update request with clone options."""
  pitr_context = request.pointInTimeRestoreContext
  if args.restore_database_names:
    pitr_context.databaseNames[:] = [args.restore_database_names]

  if args.private_network:
    pitr_context.privateNetwork = args.private_network

  if args.preferred_zone:
    pitr_context.preferredZone = args.preferred_zone

  if args.preferred_secondary_zone:
    pitr_context.preferredSecondaryZone = args.preferred_secondary_zone

  if args.allocated_ip_range_name:
    pitr_context.allocatedIpRange = args.allocated_ip_range_name

  if args.region:
    pitr_context.region = args.region


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA
)
class PointInTimeRestore(base.Command):
  """Performs a point in time restore for a Cloud SQL instance managed by Google Cloud Backup and Disaster Recovery (DR) Service."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser: parser_extensions.Namespace) -> None:
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use it to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    base.ASYNC_FLAG.AddToParser(parser)
    parser.display_info.AddFormat(flags.GetInstanceListFormat())
    parser.add_argument(
        'datasource',
        help="""\
        The Google Cloud Backup and Disaster Recovery (DR) Service Datasource URI,
        of the form projects/{project}/locations/{region}/backupVaults/
        {backupvault}/dataSources/{datasource}.
        """,
    )
    parser.add_argument(
        'target',
        help='Cloud SQL instance ID of the target instance.',
    )
    parser.add_argument(
        'point_in_time',
        type=arg_parsers.Datetime.Parse,
        help="""\
        The point in time in which to restore the instance to. Uses  RFC 3339
        format in UTC timezone. For example, '2012-11-15T16:19:00.094Z'.
        """,
    )
    parser.add_argument(
        '--private-network',
        required=False,
        help="""\
        The resource link for the VPC network from which the Cloud SQL instance is
        accessible for private IP. For example,
        \'/projects/myProject/global/networks/default\'.
        """,
    )
    parser.add_argument(
        '--allocated-ip-range-name',
        required=False,
        help="""\
        The name of the IP range allocated for the target instance with
        private network connectivity. For example:
        \'google-managed-services-default\'. If set, the target instance
        IP is created in the allocated range represented by this name.
        Reserved for future use.
        """,
    )
    parser.add_argument(
        '--preferred-zone',
        required=False,
        help="""\
        The preferred zone for the target instance. If you specify a value for
        this flag, then the target instance uses the value as the primary
        zone.
        """,
    )
    parser.add_argument(
        '--preferred-secondary-zone',
        required=False,
        help="""\
        The preferred secondary zone for the cloned regional instance. If you
        specify a value for this flag, then the target instance uses the value
        as the secondary zone. The secondary zone can't be the same as the
        primary zone.
        """,
    )
    parser.add_argument(
        '--restore-database-names',
        required=False,
        help="""\
      The name of the databases to be restored for a point-in-time restore. If
      set, the destination instance will only restore the specified databases.
      """,
    )
    flags.AddSourceInstanceOverrideArgs(parser=parser)

  def Run(self, args: parser_extensions.Namespace):
    """Performs a point in time restore for a Cloud SQL instance.

    The instance is managed by Google Cloud Backup and Disaster Recovery
    (DR) Service.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A dict object representing if the point-in-time restore operation was
      successful.

    Raises:
      ArgumentError: The arguments are invalid.
    """
    client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
    sql_client = client.sql_client
    sql_messages = client.sql_messages

    specified_args_dict = getattr(args, '_specified_args', None)
    overrides = [
        key
        for key in specified_args_dict
        if key in constants.TARGET_INSTANCE_OVERRIDE_FLAGS
    ]
    clears = [
        key
        for key in specified_args_dict
        if key in constants.TARGET_INSTANCE_CLEAR_FLAGS
    ]

    request = sql_messages.SqlInstancesPointInTimeRestoreRequest(
        parent=f'projects/{properties.VALUES.core.project.GetOrFail()}',
        pointInTimeRestoreContext=sql_messages.PointInTimeRestoreContext(
            datasource=args.datasource,
            targetInstance=args.target,
            pointInTime=args.point_in_time.strftime('%Y-%m-%dT%H:%M:%S.%fZ'),
        ),
    )
    _UpdateRequestFromArgs(request, args)
    destination_instance_ref = _GetInstanceRefFromArgs(args, client)

    # If the request has overrides, construct the target instance resource from
    # the args and pass it with the request. Note that currently, overrides are
    # only supported for cross project PITR. But here in gcloud, we do not have
    # a way to know whether the PITR is cross project because we do not know the
    # source project. So the check is done in the backend, where if the source
    # and target projects are not different, the overrides will be ignored.
    if overrides:
      instance_resource = (
          command_util.InstancesV1Beta4.ConstructCreateInstanceFromArgs(
              sql_messages, args, instance_ref=destination_instance_ref
          )
      )
      request.pointInTimeRestoreContext.targetInstanceSettings = (
          instance_resource
      )

    if clears:
      request.pointInTimeRestoreContext.targetInstanceClearSettingsFieldNames = (
          flags.GetInstanceClearOverrides(args)
      )

    response = sql_client.instances.PointInTimeRestore(request)

    operation_ref = client.resource_parser.Create(
        'sql.operations',
        operation=response.name,
        project=destination_instance_ref.project,
    )

    if args.async_:
      if not args.IsSpecified('format'):
        args.format = 'default'
      return sql_client.operations.Get(
          sql_messages.SqlOperationsGetRequest(
              project=operation_ref.project, operation=operation_ref.operation
          )
      )

    operations.OperationsV1Beta4.WaitForOperation(
        sql_client,
        operation_ref,
        'Performing point-in-time restore',
        max_wait_seconds=args.timeout,
    )
    log.CreatedResource(destination_instance_ref)
    resource = sql_client.instances.Get(
        sql_messages.SqlInstancesGetRequest(
            project=destination_instance_ref.project,
            instance=destination_instance_ref.instance,
        )
    )
    resource.kind = None
    return resource



"""Generated message classes for cloudquotas version v1beta.

Cloud Quotas API provides Google Cloud service consumers with management and
observability for resource usage, quotas, and restrictions of the services
they consume.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding


package = 'cloudquotas'


class CloudquotasFoldersLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest(_messages.Message):
  r"""A CloudquotasFoldersLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettin
  gsRequest object.

  Fields:
    name: Required. Identifier. Name of the `quotaAdjusterSettings`
      configuration. Only a single setting per project is supported.
  """

  name = _messages.StringField(1, required=True)


class CloudquotasFoldersLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest(_messages.Message):
  r"""A CloudquotasFoldersLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSet
  tingsRequest object.

  Fields:
    name: Identifier. Name of the configuration, in the formats below: * For a
      project: projects/PROJECT_NUMBER/locations/global/quotaAdjusterSettings
      * For a folder:
      folders/FOLDER_NUMBER/locations/global/quotaAdjusterSettings * For an
      organization:
      organizations/ORGANIZATION_NUMBER/locations/global/quotaAdjusterSettings
    quotaAdjusterSettings: A QuotaAdjusterSettings resource to be passed as
      the request body.
    updateMask: Optional. The list of fields to update.
    validateOnly: Optional. If set to true, checks the syntax of the request
      but doesn't update the quota adjuster settings value. Note that although
      a request can be valid, that doesn't guarantee that the request will be
      fulfilled.
  """

  name = _messages.StringField(1, required=True)
  quotaAdjusterSettings = _messages.MessageField('QuotaAdjusterSettings', 2)
  updateMask = _messages.StringField(3)
  validateOnly = _messages.BooleanField(4)


class CloudquotasFoldersLocationsQuotaPreferencesCreateRequest(_messages.Message):
  r"""A CloudquotasFoldersLocationsQuotaPreferencesCreateRequest object.

  Enums:
    IgnoreSafetyChecksValueValuesEnum: The list of quota safety checks to be
      ignored.

  Fields:
    ignoreSafetyChecks: The list of quota safety checks to be ignored.
    parent: Required. Identifier. Value for parent. Example:
      `projects/123/locations/global`
    quotaPreference: A QuotaPreference resource to be passed as the request
      body.
    quotaPreferenceId: Optional. Id of the requesting object, must be unique
      under its parent. If client does not set this field, the service will
      generate one.
  """

  class IgnoreSafetyChecksValueValuesEnum(_messages.Enum):
    r"""The list of quota safety checks to be ignored.

    Values:
      QUOTA_SAFETY_CHECK_UNSPECIFIED: Unspecified quota safety check.
      QUOTA_DECREASE_BELOW_USAGE: Validates that a quota mutation would not
        cause the consumer's effective limit to be lower than the consumer's
        quota usage.
      QUOTA_DECREASE_PERCENTAGE_TOO_HIGH: Validates that a quota mutation
        would not cause the consumer's effective limit to decrease by more
        than 10 percent.
    """
    QUOTA_SAFETY_CHECK_UNSPECIFIED = 0
    QUOTA_DECREASE_BELOW_USAGE = 1
    QUOTA_DECREASE_PERCENTAGE_TOO_HIGH = 2

  ignoreSafetyChecks = _messages.EnumField('IgnoreSafetyChecksValueValuesEnum', 1, repeated=True)
  parent = _messages.StringField(2, required=True)
  quotaPreference = _messages.MessageField('QuotaPreference', 3)
  quotaPreferenceId = _messages.StringField(4)


class CloudquotasFoldersLocationsQuotaPreferencesGetRequest(_messages.Message):
  r"""A CloudquotasFoldersLocationsQuotaPreferencesGetRequest object.

  Fields:
    name: Required. Identifier. Name of the resource Example name:
      `projects/123/locations/global/quota_preferences/my-config-for-us-east1`
  """

  name = _messages.StringField(1, required=True)


class CloudquotasFoldersLocationsQuotaPreferencesListRequest(_messages.Message):
  r"""A CloudquotasFoldersLocationsQuotaPreferencesListRequest object.

  Fields:
    filter: Optional. Filter result QuotaPreferences by their state, type,
      create/update time range. Example filters: `reconciling=true AND
      request_type=CLOUD_CONSOLE`, `reconciling=true OR
      creation_time>2022-12-03T10:30:00`
    orderBy: Optional. How to order of the results. By default, the results
      are ordered by create time. Example orders: `quota_id`, `service,
      create_time`
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Identifier. Parent value of QuotaPreference resources.
      Listing across different resource containers (such as 'projects/-') is
      not allowed. When the value starts with 'folders' or 'organizations', it
      lists the QuotaPreferences for org quotas in the container. It does not
      list the QuotaPreferences in the descendant projects of the container.
      Example parents: `projects/123/locations/global`
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class CloudquotasFoldersLocationsQuotaPreferencesPatchRequest(_messages.Message):
  r"""A CloudquotasFoldersLocationsQuotaPreferencesPatchRequest object.

  Enums:
    IgnoreSafetyChecksValueValuesEnum: The list of quota safety checks to be
      ignored.

  Fields:
    allowMissing: Optional. If set to true, and the quota preference is not
      found, a new one will be created. In this situation, `update_mask` is
      ignored.
    ignoreSafetyChecks: The list of quota safety checks to be ignored.
    name: Required except in the CREATE requests. The resource name of the
      quota preference. The path that follows `/locations` must be `/global`.
      For example: `projects/123/locations/global/quotaPreferences/my-config-
      for-us-east1`
    quotaPreference: A QuotaPreference resource to be passed as the request
      body.
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the QuotaPreference resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. A field will be overwritten if it is in the mask. If the user
      does not provide a mask then all fields will be overwritten.
    validateOnly: Optional. If set to true, validate the request, but do not
      actually update. Note that a request being valid does not mean that the
      request is guaranteed to be fulfilled.
  """

  class IgnoreSafetyChecksValueValuesEnum(_messages.Enum):
    r"""The list of quota safety checks to be ignored.

    Values:
      QUOTA_SAFETY_CHECK_UNSPECIFIED: Unspecified quota safety check.
      QUOTA_DECREASE_BELOW_USAGE: Validates that a quota mutation would not
        cause the consumer's effective limit to be lower than the consumer's
        quota usage.
      QUOTA_DECREASE_PERCENTAGE_TOO_HIGH: Validates that a quota mutation
        would not cause the consumer's effective limit to decrease by more
        than 10 percent.
    """
    QUOTA_SAFETY_CHECK_UNSPECIFIED = 0
    QUOTA_DECREASE_BELOW_USAGE = 1
    QUOTA_DECREASE_PERCENTAGE_TOO_HIGH = 2

  allowMissing = _messages.BooleanField(1)
  ignoreSafetyChecks = _messages.EnumField('IgnoreSafetyChecksValueValuesEnum', 2, repeated=True)
  name = _messages.StringField(3, required=True)
  quotaPreference = _messages.MessageField('QuotaPreference', 4)
  updateMask = _messages.StringField(5)
  validateOnly = _messages.BooleanField(6)


class CloudquotasFoldersLocationsServicesQuotaInfosGetRequest(_messages.Message):
  r"""A CloudquotasFoldersLocationsServicesQuotaInfosGetRequest object.

  Fields:
    name: Required. Identifier. The resource name of the quota info. An
      example name: `projects/123/locations/global/services/compute.googleapis
      .com/quotaInfos/CpusPerProjectPerRegion`
  """

  name = _messages.StringField(1, required=True)


class CloudquotasFoldersLocationsServicesQuotaInfosListRequest(_messages.Message):
  r"""A CloudquotasFoldersLocationsServicesQuotaInfosListRequest object.

  Fields:
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Identifier. Parent value of QuotaInfo resources. Listing
      across different resource containers (such as 'projects/-') is not
      allowed. Example names:
      `projects/123/locations/global/services/compute.googleapis.com`
      `folders/234/locations/global/services/compute.googleapis.com`
      `organizations/345/locations/global/services/compute.googleapis.com`
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudquotasOrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosGetRequest(_messages.Message):
  r"""A
  CloudquotasOrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosGetRequest
  object.

  Fields:
    name: Required. The resource name of the GdcQuotaInfo. An example name:
      `projects/123/locations/us-west2/gdcZones/us-west2-edge-
      lga1/gdcServices/edgecontainer.googleapis.com/gdcQuotaInfos/gdc-quota-1`
      `organizations/123/locations/us-west2/gdcZones/us-west2-edge-
      lga1/gdcServices/edgecontainer.googleapis.com/gdcQuotaInfos/gdc-quota-1`
  """

  name = _messages.StringField(1, required=True)


class CloudquotasOrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosListRequest(_messages.Message):
  r"""A
  CloudquotasOrganizationsLocationsGdcZonesGdcServicesGdcQuotaInfosListRequest
  object.

  Fields:
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, the default page size is 50. The maximum
      value is 1000; values above 1000 will be coerced to 1000.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Parent value of GdcQuotaInfo resources. Listing across
      different resource containers (such as 'projects/-') is not allowed.
      Example parents: `projects/123/locations/us-west2/gdcZones/us-
      west2-edge-lga1/gdcServices/edgecontainer.googleapis.com`
      `organizations/123/locations/us-west2/gdcZones/us-west2-edge-
      lga1/gdcServices/edgecontainer.googleapis.com`
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudquotasOrganizationsLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest(_messages.Message):
  r"""A CloudquotasOrganizationsLocationsQuotaAdjusterSettingsGetQuotaAdjuster
  SettingsRequest object.

  Fields:
    name: Required. Identifier. Name of the `quotaAdjusterSettings`
      configuration. Only a single setting per project is supported.
  """

  name = _messages.StringField(1, required=True)


class CloudquotasOrganizationsLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest(_messages.Message):
  r"""A CloudquotasOrganizationsLocationsQuotaAdjusterSettingsUpdateQuotaAdjus
  terSettingsRequest object.

  Fields:
    name: Identifier. Name of the configuration, in the formats below: * For a
      project: projects/PROJECT_NUMBER/locations/global/quotaAdjusterSettings
      * For a folder:
      folders/FOLDER_NUMBER/locations/global/quotaAdjusterSettings * For an
      organization:
      organizations/ORGANIZATION_NUMBER/locations/global/quotaAdjusterSettings
    quotaAdjusterSettings: A QuotaAdjusterSettings resource to be passed as
      the request body.
    updateMask: Optional. The list of fields to update.
    validateOnly: Optional. If set to true, checks the syntax of the request
      but doesn't update the quota adjuster settings value. Note that although
      a request can be valid, that doesn't guarantee that the request will be
      fulfilled.
  """

  name = _messages.StringField(1, required=True)
  quotaAdjusterSettings = _messages.MessageField('QuotaAdjusterSettings', 2)
  updateMask = _messages.StringField(3)
  validateOnly = _messages.BooleanField(4)


class CloudquotasOrganizationsLocationsQuotaPreferencesCreateRequest(_messages.Message):
  r"""A CloudquotasOrganizationsLocationsQuotaPreferencesCreateRequest object.

  Enums:
    IgnoreSafetyChecksValueValuesEnum: The list of quota safety checks to be
      ignored.

  Fields:
    ignoreSafetyChecks: The list of quota safety checks to be ignored.
    parent: Required. Identifier. Value for parent. Example:
      `projects/123/locations/global`
    quotaPreference: A QuotaPreference resource to be passed as the request
      body.
    quotaPreferenceId: Optional. Id of the requesting object, must be unique
      under its parent. If client does not set this field, the service will
      generate one.
  """

  class IgnoreSafetyChecksValueValuesEnum(_messages.Enum):
    r"""The list of quota safety checks to be ignored.

    Values:
      QUOTA_SAFETY_CHECK_UNSPECIFIED: Unspecified quota safety check.
      QUOTA_DECREASE_BELOW_USAGE: Validates that a quota mutation would not
        cause the consumer's effective limit to be lower than the consumer's
        quota usage.
      QUOTA_DECREASE_PERCENTAGE_TOO_HIGH: Validates that a quota mutation
        would not cause the consumer's effective limit to decrease by more
        than 10 percent.
    """
    QUOTA_SAFETY_CHECK_UNSPECIFIED = 0
    QUOTA_DECREASE_BELOW_USAGE = 1
    QUOTA_DECREASE_PERCENTAGE_TOO_HIGH = 2

  ignoreSafetyChecks = _messages.EnumField('IgnoreSafetyChecksValueValuesEnum', 1, repeated=True)
  parent = _messages.StringField(2, required=True)
  quotaPreference = _messages.MessageField('QuotaPreference', 3)
  quotaPreferenceId = _messages.StringField(4)


class CloudquotasOrganizationsLocationsQuotaPreferencesGetRequest(_messages.Message):
  r"""A CloudquotasOrganizationsLocationsQuotaPreferencesGetRequest object.

  Fields:
    name: Required. Identifier. Name of the resource Example name:
      `projects/123/locations/global/quota_preferences/my-config-for-us-east1`
  """

  name = _messages.StringField(1, required=True)


class CloudquotasOrganizationsLocationsQuotaPreferencesListRequest(_messages.Message):
  r"""A CloudquotasOrganizationsLocationsQuotaPreferencesListRequest object.

  Fields:
    filter: Optional. Filter result QuotaPreferences by their state, type,
      create/update time range. Example filters: `reconciling=true AND
      request_type=CLOUD_CONSOLE`, `reconciling=true OR
      creation_time>2022-12-03T10:30:00`
    orderBy: Optional. How to order of the results. By default, the results
      are ordered by create time. Example orders: `quota_id`, `service,
      create_time`
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Identifier. Parent value of QuotaPreference resources.
      Listing across different resource containers (such as 'projects/-') is
      not allowed. When the value starts with 'folders' or 'organizations', it
      lists the QuotaPreferences for org quotas in the container. It does not
      list the QuotaPreferences in the descendant projects of the container.
      Example parents: `projects/123/locations/global`
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class CloudquotasOrganizationsLocationsQuotaPreferencesPatchRequest(_messages.Message):
  r"""A CloudquotasOrganizationsLocationsQuotaPreferencesPatchRequest object.

  Enums:
    IgnoreSafetyChecksValueValuesEnum: The list of quota safety checks to be
      ignored.

  Fields:
    allowMissing: Optional. If set to true, and the quota preference is not
      found, a new one will be created. In this situation, `update_mask` is
      ignored.
    ignoreSafetyChecks: The list of quota safety checks to be ignored.
    name: Required except in the CREATE requests. The resource name of the
      quota preference. The path that follows `/locations` must be `/global`.
      For example: `projects/123/locations/global/quotaPreferences/my-config-
      for-us-east1`
    quotaPreference: A QuotaPreference resource to be passed as the request
      body.
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the QuotaPreference resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. A field will be overwritten if it is in the mask. If the user
      does not provide a mask then all fields will be overwritten.
    validateOnly: Optional. If set to true, validate the request, but do not
      actually update. Note that a request being valid does not mean that the
      request is guaranteed to be fulfilled.
  """

  class IgnoreSafetyChecksValueValuesEnum(_messages.Enum):
    r"""The list of quota safety checks to be ignored.

    Values:
      QUOTA_SAFETY_CHECK_UNSPECIFIED: Unspecified quota safety check.
      QUOTA_DECREASE_BELOW_USAGE: Validates that a quota mutation would not
        cause the consumer's effective limit to be lower than the consumer's
        quota usage.
      QUOTA_DECREASE_PERCENTAGE_TOO_HIGH: Validates that a quota mutation
        would not cause the consumer's effective limit to decrease by more
        than 10 percent.
    """
    QUOTA_SAFETY_CHECK_UNSPECIFIED = 0
    QUOTA_DECREASE_BELOW_USAGE = 1
    QUOTA_DECREASE_PERCENTAGE_TOO_HIGH = 2

  allowMissing = _messages.BooleanField(1)
  ignoreSafetyChecks = _messages.EnumField('IgnoreSafetyChecksValueValuesEnum', 2, repeated=True)
  name = _messages.StringField(3, required=True)
  quotaPreference = _messages.MessageField('QuotaPreference', 4)
  updateMask = _messages.StringField(5)
  validateOnly = _messages.BooleanField(6)


class CloudquotasOrganizationsLocationsServicesQuotaInfosGetRequest(_messages.Message):
  r"""A CloudquotasOrganizationsLocationsServicesQuotaInfosGetRequest object.

  Fields:
    name: Required. Identifier. The resource name of the quota info. An
      example name: `projects/123/locations/global/services/compute.googleapis
      .com/quotaInfos/CpusPerProjectPerRegion`
  """

  name = _messages.StringField(1, required=True)


class CloudquotasOrganizationsLocationsServicesQuotaInfosListRequest(_messages.Message):
  r"""A CloudquotasOrganizationsLocationsServicesQuotaInfosListRequest object.

  Fields:
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Identifier. Parent value of QuotaInfo resources. Listing
      across different resource containers (such as 'projects/-') is not
      allowed. Example names:
      `projects/123/locations/global/services/compute.googleapis.com`
      `folders/234/locations/global/services/compute.googleapis.com`
      `organizations/345/locations/global/services/compute.googleapis.com`
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudquotasProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosGetRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosGetRequest
  object.

  Fields:
    name: Required. The resource name of the GdcQuotaInfo. An example name:
      `projects/123/locations/us-west2/gdcZones/us-west2-edge-
      lga1/gdcServices/edgecontainer.googleapis.com/gdcQuotaInfos/gdc-quota-1`
      `organizations/123/locations/us-west2/gdcZones/us-west2-edge-
      lga1/gdcServices/edgecontainer.googleapis.com/gdcQuotaInfos/gdc-quota-1`
  """

  name = _messages.StringField(1, required=True)


class CloudquotasProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosListRequest(_messages.Message):
  r"""A
  CloudquotasProjectsLocationsGdcZonesGdcServicesGdcQuotaInfosListRequest
  object.

  Fields:
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, the default page size is 50. The maximum
      value is 1000; values above 1000 will be coerced to 1000.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Parent value of GdcQuotaInfo resources. Listing across
      different resource containers (such as 'projects/-') is not allowed.
      Example parents: `projects/123/locations/us-west2/gdcZones/us-
      west2-edge-lga1/gdcServices/edgecontainer.googleapis.com`
      `organizations/123/locations/us-west2/gdcZones/us-west2-edge-
      lga1/gdcServices/edgecontainer.googleapis.com`
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class CloudquotasProjectsLocationsQuotaAdjusterSettingsGetQuotaAdjusterSettingsRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsQuotaAdjusterSettingsGetQuotaAdjusterSetti
  ngsRequest object.

  Fields:
    name: Required. Identifier. Name of the `quotaAdjusterSettings`
      configuration. Only a single setting per project is supported.
  """

  name = _messages.StringField(1, required=True)


class CloudquotasProjectsLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSettingsRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsQuotaAdjusterSettingsUpdateQuotaAdjusterSe
  ttingsRequest object.

  Fields:
    name: Identifier. Name of the configuration, in the formats below: * For a
      project: projects/PROJECT_NUMBER/locations/global/quotaAdjusterSettings
      * For a folder:
      folders/FOLDER_NUMBER/locations/global/quotaAdjusterSettings * For an
      organization:
      organizations/ORGANIZATION_NUMBER/locations/global/quotaAdjusterSettings
    quotaAdjusterSettings: A QuotaAdjusterSettings resource to be passed as
      the request body.
    updateMask: Optional. The list of fields to update.
    validateOnly: Optional. If set to true, checks the syntax of the request
      but doesn't update the quota adjuster settings value. Note that although
      a request can be valid, that doesn't guarantee that the request will be
      fulfilled.
  """

  name = _messages.StringField(1, required=True)
  quotaAdjusterSettings = _messages.MessageField('QuotaAdjusterSettings', 2)
  updateMask = _messages.StringField(3)
  validateOnly = _messages.BooleanField(4)


class CloudquotasProjectsLocationsQuotaPreferencesCreateRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsQuotaPreferencesCreateRequest object.

  Enums:
    IgnoreSafetyChecksValueValuesEnum: The list of quota safety checks to be
      ignored.

  Fields:
    ignoreSafetyChecks: The list of quota safety checks to be ignored.
    parent: Required. Identifier. Value for parent. Example:
      `projects/123/locations/global`
    quotaPreference: A QuotaPreference resource to be passed as the request
      body.
    quotaPreferenceId: Optional. Id of the requesting object, must be unique
      under its parent. If client does not set this field, the service will
      generate one.
  """

  class IgnoreSafetyChecksValueValuesEnum(_messages.Enum):
    r"""The list of quota safety checks to be ignored.

    Values:
      QUOTA_SAFETY_CHECK_UNSPECIFIED: Unspecified quota safety check.
      QUOTA_DECREASE_BELOW_USAGE: Validates that a quota mutation would not
        cause the consumer's effective limit to be lower than the consumer's
        quota usage.
      QUOTA_DECREASE_PERCENTAGE_TOO_HIGH: Validates that a quota mutation
        would not cause the consumer's effective limit to decrease by more
        than 10 percent.
    """
    QUOTA_SAFETY_CHECK_UNSPECIFIED = 0
    QUOTA_DECREASE_BELOW_USAGE = 1
    QUOTA_DECREASE_PERCENTAGE_TOO_HIGH = 2

  ignoreSafetyChecks = _messages.EnumField('IgnoreSafetyChecksValueValuesEnum', 1, repeated=True)
  parent = _messages.StringField(2, required=True)
  quotaPreference = _messages.MessageField('QuotaPreference', 3)
  quotaPreferenceId = _messages.StringField(4)


class CloudquotasProjectsLocationsQuotaPreferencesGetRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsQuotaPreferencesGetRequest object.

  Fields:
    name: Required. Identifier. Name of the resource Example name:
      `projects/123/locations/global/quota_preferences/my-config-for-us-east1`
  """

  name = _messages.StringField(1, required=True)


class CloudquotasProjectsLocationsQuotaPreferencesListRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsQuotaPreferencesListRequest object.

  Fields:
    filter: Optional. Filter result QuotaPreferences by their state, type,
      create/update time range. Example filters: `reconciling=true AND
      request_type=CLOUD_CONSOLE`, `reconciling=true OR
      creation_time>2022-12-03T10:30:00`
    orderBy: Optional. How to order of the results. By default, the results
      are ordered by create time. Example orders: `quota_id`, `service,
      create_time`
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Identifier. Parent value of QuotaPreference resources.
      Listing across different resource containers (such as 'projects/-') is
      not allowed. When the value starts with 'folders' or 'organizations', it
      lists the QuotaPreferences for org quotas in the container. It does not
      list the QuotaPreferences in the descendant projects of the container.
      Example parents: `projects/123/locations/global`
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class CloudquotasProjectsLocationsQuotaPreferencesPatchRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsQuotaPreferencesPatchRequest object.

  Enums:
    IgnoreSafetyChecksValueValuesEnum: The list of quota safety checks to be
      ignored.

  Fields:
    allowMissing: Optional. If set to true, and the quota preference is not
      found, a new one will be created. In this situation, `update_mask` is
      ignored.
    ignoreSafetyChecks: The list of quota safety checks to be ignored.
    name: Required except in the CREATE requests. The resource name of the
      quota preference. The path that follows `/locations` must be `/global`.
      For example: `projects/123/locations/global/quotaPreferences/my-config-
      for-us-east1`
    quotaPreference: A QuotaPreference resource to be passed as the request
      body.
    updateMask: Optional. Field mask is used to specify the fields to be
      overwritten in the QuotaPreference resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. A field will be overwritten if it is in the mask. If the user
      does not provide a mask then all fields will be overwritten.
    validateOnly: Optional. If set to true, validate the request, but do not
      actually update. Note that a request being valid does not mean that the
      request is guaranteed to be fulfilled.
  """

  class IgnoreSafetyChecksValueValuesEnum(_messages.Enum):
    r"""The list of quota safety checks to be ignored.

    Values:
      QUOTA_SAFETY_CHECK_UNSPECIFIED: Unspecified quota safety check.
      QUOTA_DECREASE_BELOW_USAGE: Validates that a quota mutation would not
        cause the consumer's effective limit to be lower than the consumer's
        quota usage.
      QUOTA_DECREASE_PERCENTAGE_TOO_HIGH: Validates that a quota mutation
        would not cause the consumer's effective limit to decrease by more
        than 10 percent.
    """
    QUOTA_SAFETY_CHECK_UNSPECIFIED = 0
    QUOTA_DECREASE_BELOW_USAGE = 1
    QUOTA_DECREASE_PERCENTAGE_TOO_HIGH = 2

  allowMissing = _messages.BooleanField(1)
  ignoreSafetyChecks = _messages.EnumField('IgnoreSafetyChecksValueValuesEnum', 2, repeated=True)
  name = _messages.StringField(3, required=True)
  quotaPreference = _messages.MessageField('QuotaPreference', 4)
  updateMask = _messages.StringField(5)
  validateOnly = _messages.BooleanField(6)


class CloudquotasProjectsLocationsServicesQuotaInfosGetRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsServicesQuotaInfosGetRequest object.

  Fields:
    name: Required. Identifier. The resource name of the quota info. An
      example name: `projects/123/locations/global/services/compute.googleapis
      .com/quotaInfos/CpusPerProjectPerRegion`
  """

  name = _messages.StringField(1, required=True)


class CloudquotasProjectsLocationsServicesQuotaInfosListRequest(_messages.Message):
  r"""A CloudquotasProjectsLocationsServicesQuotaInfosListRequest object.

  Fields:
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Identifier. Parent value of QuotaInfo resources. Listing
      across different resource containers (such as 'projects/-') is not
      allowed. Example names:
      `projects/123/locations/global/services/compute.googleapis.com`
      `folders/234/locations/global/services/compute.googleapis.com`
      `organizations/345/locations/global/services/compute.googleapis.com`
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class DimensionsInfo(_messages.Message):
  r"""The detailed quota information such as effective quota value for a
  combination of dimensions.

  Messages:
    DimensionsValue: The map of dimensions in key-value pairs. The key of a
      map entry is "region", "zone", or the name of a service-specific
      dimension, and the value of a map entry is the value of the dimension.
      If a dimension does not appear in the map of dimensions, the dimensions
      info applies to all the dimension values except for those that have
      another DimensionInfo instance configured for the specific value. For
      example: `{"provider" : "Example Organization"}` where `provider` is a
      service-specific quota dimension and `Example Organization` is the
      provider name.

  Fields:
    applicableLocations: The applicable regions or zones of this dimension.
      The field is set to ['global'] for quotas that are not per region or per
      zone. Otherwise, it will be set to the list of locations this dimension
      info is applicable to.
    details: Quota details for the specified dimensions.
    dimensions: The map of dimensions in key-value pairs. The key of a map
      entry is "region", "zone", or the name of a service-specific dimension,
      and the value of a map entry is the value of the dimension. If a
      dimension does not appear in the map of dimensions, the dimensions info
      applies to all the dimension values except for those that have another
      DimensionInfo instance configured for the specific value. For example:
      `{"provider" : "Example Organization"}` where `provider` is a service-
      specific quota dimension and `Example Organization` is the provider
      name.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DimensionsValue(_messages.Message):
    r"""The map of dimensions in key-value pairs. The key of a map entry is
    "region", "zone", or the name of a service-specific dimension, and the
    value of a map entry is the value of the dimension. If a dimension does
    not appear in the map of dimensions, the dimensions info applies to all
    the dimension values except for those that have another DimensionInfo
    instance configured for the specific value. For example: `{"provider" :
    "Example Organization"}` where `provider` is a service-specific quota
    dimension and `Example Organization` is the provider name.

    Messages:
      AdditionalProperty: An additional property for a DimensionsValue object.

    Fields:
      additionalProperties: Additional properties of type DimensionsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DimensionsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  applicableLocations = _messages.StringField(1, repeated=True)
  details = _messages.MessageField('QuotaDetails', 2)
  dimensions = _messages.MessageField('DimensionsValue', 3)


class GdcDimensionsInfo(_messages.Message):
  r"""The detailed quota information such as effective quota value for a
  combination of dimensions.

  Messages:
    DimensionsValue: The map of dimensions in key-value pairs. The key of a
      map entry is "region", "zone", or the name of a service-specific
      dimension, and the value of a map entry is the value of the dimension.
      If a dimension does not appear in the map of dimensions, the dimensions
      info applies to all the dimension values except for those that have
      another DimensionInfo instance configured for the specific value. For
      example: `{"provider" : "Example Organization"}` where `provider` is a
      service-specific quota dimension and `Example Organization` is the
      provider name.

  Fields:
    details: Quota details for the specified dimensions.
    dimensions: The map of dimensions in key-value pairs. The key of a map
      entry is "region", "zone", or the name of a service-specific dimension,
      and the value of a map entry is the value of the dimension. If a
      dimension does not appear in the map of dimensions, the dimensions info
      applies to all the dimension values except for those that have another
      DimensionInfo instance configured for the specific value. For example:
      `{"provider" : "Example Organization"}` where `provider` is a service-
      specific quota dimension and `Example Organization` is the provider
      name.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DimensionsValue(_messages.Message):
    r"""The map of dimensions in key-value pairs. The key of a map entry is
    "region", "zone", or the name of a service-specific dimension, and the
    value of a map entry is the value of the dimension. If a dimension does
    not appear in the map of dimensions, the dimensions info applies to all
    the dimension values except for those that have another DimensionInfo
    instance configured for the specific value. For example: `{"provider" :
    "Example Organization"}` where `provider` is a service-specific quota
    dimension and `Example Organization` is the provider name.

    Messages:
      AdditionalProperty: An additional property for a DimensionsValue object.

    Fields:
      additionalProperties: Additional properties of type DimensionsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DimensionsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  details = _messages.MessageField('GdcQuotaDetails', 1)
  dimensions = _messages.MessageField('DimensionsValue', 2)


class GdcQuotaDetails(_messages.Message):
  r"""The quota details for a map of dimensions.

  Fields:
    resetValue: The Google Cloud defined quota value that the quota will be
      reset to if a quota decrease preference is deleted
    value: The value currently in effect and being enforced.
  """

  resetValue = _messages.IntegerField(1)
  value = _messages.IntegerField(2)


class GdcQuotaInfo(_messages.Message):
  r"""GdcQuotaInfo represents information about a particular quota for a given
  project, folder or organization in GDC.

  Enums:
    ContainerTypeValueValuesEnum: Output only. The type of a cloud resource
      container that the quota is defined for.

  Fields:
    containerType: Output only. The type of a cloud resource container that
      the quota is defined for.
    dimensions: Output only. The dimensions the quota is defined on.
    gdcDimensionsInfos: Output only. The collection of dimensions info ordered
      by their dimensions from more specific ones to less specific ones.
    metric: Output only. The metric of the quota. It specifies the resources
      consumption the quota is defined for. For example,
      `compute.googleapis.com/cpus`
    name: Output only. Identifier. Resource name of this GdcQuotaInfo. For
      example, `projects/123/locations/us-west2/gdcZones/us-west2-edge-
      lga1/gdcServices/edgecontainer.googleapis.com/gdcQuotaInfos/gdc-quota-1`
      `organizations/123/locations/us-west2/gdcZones/us-west2-edge-
      lga1/gdcServices/edgecontainer.googleapis.com/gdcQuotaInfos/gdc-quota-1`
    precise: Output only. Whether this is a precise quota. A precise quota is
      tracked with absolute precision. In contrast, an imprecise quota is not
      tracked with precision.
    quotaId: Output only. The id of the quota, which is unique within the
      service. For example, `CpusPerProjectPerRegion`
    refreshInterval: Output only. The reset time interval for the quota.
      Refresh interval applies to rate quota only. For example, "minute" for
      per minute, "day" for per day, or "10 seconds" for every 10 seconds.
    service: Output only. The name of the service in which the quota is
      defined. For example, `compute.googleapis.com`
  """

  class ContainerTypeValueValuesEnum(_messages.Enum):
    r"""Output only. The type of a cloud resource container that the quota is
    defined for.

    Values:
      GDC_CONTAINER_TYPE_UNSPECIFIED: Unspecified container type.
      GDC_PROJECT: consumer project
      GDC_ORGANIZATION: organization
    """
    GDC_CONTAINER_TYPE_UNSPECIFIED = 0
    GDC_PROJECT = 1
    GDC_ORGANIZATION = 2

  containerType = _messages.EnumField('ContainerTypeValueValuesEnum', 1)
  dimensions = _messages.StringField(2, repeated=True)
  gdcDimensionsInfos = _messages.MessageField('GdcDimensionsInfo', 3, repeated=True)
  metric = _messages.StringField(4)
  name = _messages.StringField(5)
  precise = _messages.BooleanField(6)
  quotaId = _messages.StringField(7)
  refreshInterval = _messages.StringField(8)
  service = _messages.StringField(9)


class ListGdcQuotaInfosResponse(_messages.Message):
  r"""Message for response to listing GdcQuotaInfos

  Fields:
    gdcQuotaInfos: The list of GdcQuotaInfo
    nextPageToken: A token identifying a page of results the server should
      return.
  """

  gdcQuotaInfos = _messages.MessageField('GdcQuotaInfo', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListQuotaInfosResponse(_messages.Message):
  r"""Message for response to listing QuotaInfos

  Fields:
    nextPageToken: A token, which can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    quotaInfos: The list of QuotaInfo
  """

  nextPageToken = _messages.StringField(1)
  quotaInfos = _messages.MessageField('QuotaInfo', 2, repeated=True)


class ListQuotaPreferencesResponse(_messages.Message):
  r"""Message for response to listing QuotaPreferences

  Fields:
    nextPageToken: A token, which can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    quotaPreferences: The list of QuotaPreference
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  quotaPreferences = _messages.MessageField('QuotaPreference', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    cancelRequested: Output only. Identifies whether the user has requested
      cancellation of the operation. Operations that have been cancelled
      successfully have google.longrunning.Operation.error value with a
      google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    statusDetail: Output only. Human-readable status of the operation, if any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  cancelRequested = _messages.BooleanField(2)
  createTime = _messages.StringField(3)
  endTime = _messages.StringField(4)
  statusDetail = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class QuotaAdjusterSettings(_messages.Message):
  r"""The QuotaAdjusterSettings resource defines the settings for the Quota
  Adjuster.

  Enums:
    EnablementValueValuesEnum: Optional. The configured value of the
      enablement at the given resource.

  Fields:
    enablement: Optional. The configured value of the enablement at the given
      resource.
    etag: Optional. The current ETag of the QuotaAdjusterSettings. If an ETag
      is provided on update and does not match the current server's ETag in
      the QuotaAdjusterSettings, the request is blocked and returns an ABORTED
      error. See https://google.aip.dev/134#etags for more details on ETags.
    inherited: Optional. Indicates whether the setting is inherited or
      explicitly specified.
    inheritedFrom: Output only. The resource container from which the setting
      is inherited. This refers to the nearest ancestor with enablement set
      (either ENABLED or DISABLED). The value can be an
      organizations/{organization_id}, folders/{folder_id}, or can be
      'default' if no ancestor exists with enablement set. The value will be
      empty when enablement is directly set on this container.
    name: Identifier. Name of the configuration, in the formats below: * For a
      project: projects/PROJECT_NUMBER/locations/global/quotaAdjusterSettings
      * For a folder:
      folders/FOLDER_NUMBER/locations/global/quotaAdjusterSettings * For an
      organization:
      organizations/ORGANIZATION_NUMBER/locations/global/quotaAdjusterSettings
    updateTime: Output only. The timestamp when the QuotaAdjusterSettings
      resource was last updated.
  """

  class EnablementValueValuesEnum(_messages.Enum):
    r"""Optional. The configured value of the enablement at the given
    resource.

    Values:
      ENABLEMENT_UNSPECIFIED: The quota adjuster is in an unknown state.
      ENABLED: The quota adjuster is enabled.
      DISABLED: The quota adjuster is disabled.
    """
    ENABLEMENT_UNSPECIFIED = 0
    ENABLED = 1
    DISABLED = 2

  enablement = _messages.EnumField('EnablementValueValuesEnum', 1)
  etag = _messages.StringField(2)
  inherited = _messages.BooleanField(3)
  inheritedFrom = _messages.StringField(4)
  name = _messages.StringField(5)
  updateTime = _messages.StringField(6)


class QuotaConfig(_messages.Message):
  r"""The preferred quota configuration.

  Enums:
    RequestOriginValueValuesEnum: Output only. The origin of the quota
      preference request.

  Messages:
    AnnotationsValue: Optional. The annotations map for clients to store small
      amounts of arbitrary data. Do not put PII or other sensitive information
      here. See https://google.aip.dev/128#annotations

  Fields:
    annotations: Optional. The annotations map for clients to store small
      amounts of arbitrary data. Do not put PII or other sensitive information
      here. See https://google.aip.dev/128#annotations
    grantedValue: Output only. Granted quota value.
    preferredValue: Required. The preferred value. Must be greater than or
      equal to -1. If set to -1, it means the value is "unlimited".
    requestOrigin: Output only. The origin of the quota preference request.
    stateDetail: Output only. Optional details about the state of this quota
      preference.
    traceId: Output only. The trace id that the Google Cloud uses to provision
      the requested quota. This trace id may be used by the client to contact
      Cloud support to track the state of a quota preference request. The
      trace id is only produced for increase requests and is unique for each
      request. The quota decrease requests do not have a trace id.
  """

  class RequestOriginValueValuesEnum(_messages.Enum):
    r"""Output only. The origin of the quota preference request.

    Values:
      ORIGIN_UNSPECIFIED: The unspecified value.
      CLOUD_CONSOLE: Created through Cloud Console.
      AUTO_ADJUSTER: Generated by automatic quota adjustment.
    """
    ORIGIN_UNSPECIFIED = 0
    CLOUD_CONSOLE = 1
    AUTO_ADJUSTER = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AnnotationsValue(_messages.Message):
    r"""Optional. The annotations map for clients to store small amounts of
    arbitrary data. Do not put PII or other sensitive information here. See
    https://google.aip.dev/128#annotations

    Messages:
      AdditionalProperty: An additional property for a AnnotationsValue
        object.

    Fields:
      additionalProperties: Additional properties of type AnnotationsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AnnotationsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  annotations = _messages.MessageField('AnnotationsValue', 1)
  grantedValue = _messages.IntegerField(2)
  preferredValue = _messages.IntegerField(3)
  requestOrigin = _messages.EnumField('RequestOriginValueValuesEnum', 4)
  stateDetail = _messages.StringField(5)
  traceId = _messages.StringField(6)


class QuotaDetails(_messages.Message):
  r"""The quota details for a map of dimensions.

  Fields:
    rolloutInfo: Rollout information of this quota. This field is present only
      if the effective limit will change due to the ongoing rollout of the
      service config.
    value: The value currently in effect and being enforced.
  """

  rolloutInfo = _messages.MessageField('RolloutInfo', 1)
  value = _messages.IntegerField(2)


class QuotaIncreaseEligibility(_messages.Message):
  r"""Eligibility information regarding requesting increase adjustment of a
  quota.

  Enums:
    IneligibilityReasonValueValuesEnum: The reason of why it is ineligible to
      request increased value of the quota. If the is_eligible field is true,
      it defaults to INELIGIBILITY_REASON_UNSPECIFIED.

  Fields:
    ineligibilityReason: The reason of why it is ineligible to request
      increased value of the quota. If the is_eligible field is true, it
      defaults to INELIGIBILITY_REASON_UNSPECIFIED.
    isEligible: Whether a higher quota value can be requested for the quota.
  """

  class IneligibilityReasonValueValuesEnum(_messages.Enum):
    r"""The reason of why it is ineligible to request increased value of the
    quota. If the is_eligible field is true, it defaults to
    INELIGIBILITY_REASON_UNSPECIFIED.

    Values:
      INELIGIBILITY_REASON_UNSPECIFIED: Default value when is_eligible is
        true.
      NO_VALID_BILLING_ACCOUNT: The container is not linked with a valid
        billing account.
      NOT_SUPPORTED: Quota increase is not supported for the quota.
      NOT_ENOUGH_USAGE_HISTORY: There is not enough usage history to determine
        the eligibility.
      OTHER: Other reasons.
    """
    INELIGIBILITY_REASON_UNSPECIFIED = 0
    NO_VALID_BILLING_ACCOUNT = 1
    NOT_SUPPORTED = 2
    NOT_ENOUGH_USAGE_HISTORY = 3
    OTHER = 4

  ineligibilityReason = _messages.EnumField('IneligibilityReasonValueValuesEnum', 1)
  isEligible = _messages.BooleanField(2)


class QuotaInfo(_messages.Message):
  r"""QuotaInfo represents information about a particular quota for a given
  project, folder or organization.

  Enums:
    ContainerTypeValueValuesEnum: The container type of the QuotaInfo.

  Fields:
    containerType: The container type of the QuotaInfo.
    dimensions: The dimensions the quota is defined on.
    dimensionsInfos: The collection of dimensions info ordered by their
      dimensions from more specific ones to less specific ones.
    isConcurrent: Whether the quota is a concurrent quota. Concurrent quotas
      are enforced on the total number of concurrent operations in flight at
      any given time.
    isFixed: Whether the quota value is fixed or adjustable
    isPrecise: Whether this is a precise quota. A precise quota is tracked
      with absolute precision. In contrast, an imprecise quota is not tracked
      with precision.
    metric: The metric of the quota. It specifies the resources consumption
      the quota is defined for. For example, `compute.googleapis.com/cpus`
    metricDisplayName: The display name of the quota metric
    metricUnit: The unit in which the metric value is reported, e.g., "MByte".
    name: Resource name of this QuotaInfo. The ID component following
      "locations/" must be "global". For example, `projects/123/locations/glob
      al/services/compute.googleapis.com/quotaInfos/CpusPerProjectPerRegion`
    quotaDisplayName: The display name of the quota.
    quotaId: The id of the quota, which is unique within the service. For
      example, `CpusPerProjectPerRegion`
    quotaIncreaseEligibility: Whether it is eligible to request a higher quota
      value for this quota.
    refreshInterval: The reset time interval for the quota. Refresh interval
      applies to rate quota only. For example, "minute" for per minute, "day"
      for per day, or "10 seconds" for every 10 seconds.
    service: The name of the service in which the quota is defined. For
      example, `compute.googleapis.com`
    serviceRequestQuotaUri: URI to the page where users can request more quota
      for the cloud service-for example, https://console.cloud.google.com/iam-
      admin/quotas.
  """

  class ContainerTypeValueValuesEnum(_messages.Enum):
    r"""The container type of the QuotaInfo.

    Values:
      CONTAINER_TYPE_UNSPECIFIED: Unspecified container type.
      PROJECT: consumer project
      FOLDER: folder
      ORGANIZATION: organization
    """
    CONTAINER_TYPE_UNSPECIFIED = 0
    PROJECT = 1
    FOLDER = 2
    ORGANIZATION = 3

  containerType = _messages.EnumField('ContainerTypeValueValuesEnum', 1)
  dimensions = _messages.StringField(2, repeated=True)
  dimensionsInfos = _messages.MessageField('DimensionsInfo', 3, repeated=True)
  isConcurrent = _messages.BooleanField(4)
  isFixed = _messages.BooleanField(5)
  isPrecise = _messages.BooleanField(6)
  metric = _messages.StringField(7)
  metricDisplayName = _messages.StringField(8)
  metricUnit = _messages.StringField(9)
  name = _messages.StringField(10)
  quotaDisplayName = _messages.StringField(11)
  quotaId = _messages.StringField(12)
  quotaIncreaseEligibility = _messages.MessageField('QuotaIncreaseEligibility', 13)
  refreshInterval = _messages.StringField(14)
  service = _messages.StringField(15)
  serviceRequestQuotaUri = _messages.StringField(16)


class QuotaPreference(_messages.Message):
  r"""QuotaPreference represents the preferred quota configuration specified
  for a project, folder or organization. There is only one QuotaPreference
  resource for a quota value targeting a unique set of dimensions.

  Messages:
    DimensionsValue: Immutable. The dimensions that this quota preference
      applies to. The key of the map entry is the name of a dimension, such as
      `region`, `zone`, `network_id`, and the value of the map entry is the
      dimension value. If a dimension is missing from the map of dimensions,
      the quota preference applies to all the dimension values except for
      those that have other quota preferences configured for the specific
      value. Note: QuotaPreferences can only be applied across all values of
      `user` and `resource` dimension. Do not set values for `user` or
      `resource` in the dimension map. For example: `{"provider" : "Example
      Organization"}` where `provider` is a service-specific quota dimension
      and `Example Organization` is the provider name.

  Fields:
    contactEmail: Input only. An email address that can be used to contact the
      user, in case Google Cloud needs more information to make a decision
      before additional quota can be granted. When requesting a quota
      increase, the email address is required. When requesting a quota
      decrease, the email address is optional. For example, the email address
      is optional when the `QuotaConfig.preferred_value` is smaller than the
      `QuotaDetails.reset_value`.
    createTime: Output only. Create time stamp
    dimensions: Immutable. The dimensions that this quota preference applies
      to. The key of the map entry is the name of a dimension, such as
      `region`, `zone`, `network_id`, and the value of the map entry is the
      dimension value. If a dimension is missing from the map of dimensions,
      the quota preference applies to all the dimension values except for
      those that have other quota preferences configured for the specific
      value. Note: QuotaPreferences can only be applied across all values of
      `user` and `resource` dimension. Do not set values for `user` or
      `resource` in the dimension map. For example: `{"provider" : "Example
      Organization"}` where `provider` is a service-specific quota dimension
      and `Example Organization` is the provider name.
    etag: Optional. The current etag of the quota preference. If an etag is
      provided on update and does not match the current server's etag of the
      quota preference, the request will be blocked and an ABORTED error will
      be returned. See https://google.aip.dev/134#etags for more details on
      etags.
    justification: The reason / justification for this quota preference.
    name: Required except in the CREATE requests. The resource name of the
      quota preference. The path that follows `/locations` must be `/global`.
      For example: `projects/123/locations/global/quotaPreferences/my-config-
      for-us-east1`
    quotaConfig: Required. Preferred quota configuration.
    quotaId: Required. The id of the quota to which the quota preference is
      applied. A quota name is unique in the service. For example,
      `CpusPerProjectPerRegion`
    reconciling: Output only. Is the quota preference pending Google Cloud
      approval and fulfillment.
    service: Required. The name of the service to which the quota preference
      is applied.
    updateTime: Output only. Update time stamp
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DimensionsValue(_messages.Message):
    r"""Immutable. The dimensions that this quota preference applies to. The
    key of the map entry is the name of a dimension, such as `region`, `zone`,
    `network_id`, and the value of the map entry is the dimension value. If a
    dimension is missing from the map of dimensions, the quota preference
    applies to all the dimension values except for those that have other quota
    preferences configured for the specific value. Note: QuotaPreferences can
    only be applied across all values of `user` and `resource` dimension. Do
    not set values for `user` or `resource` in the dimension map. For example:
    `{"provider" : "Example Organization"}` where `provider` is a service-
    specific quota dimension and `Example Organization` is the provider name.

    Messages:
      AdditionalProperty: An additional property for a DimensionsValue object.

    Fields:
      additionalProperties: Additional properties of type DimensionsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DimensionsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  contactEmail = _messages.StringField(1)
  createTime = _messages.StringField(2)
  dimensions = _messages.MessageField('DimensionsValue', 3)
  etag = _messages.StringField(4)
  justification = _messages.StringField(5)
  name = _messages.StringField(6)
  quotaConfig = _messages.MessageField('QuotaConfig', 7)
  quotaId = _messages.StringField(8)
  reconciling = _messages.BooleanField(9)
  service = _messages.StringField(10)
  updateTime = _messages.StringField(11)


class RolloutInfo(_messages.Message):
  r"""[Output only] Rollout information of a quota.

  Fields:
    ongoingRollout: Whether there is an ongoing rollout for a quota or not.
  """

  ongoingRollout = _messages.BooleanField(1)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to set attestation rules on a workload identity pool namespace."""


from apitools.base.py import encoding
from googlecloudsdk.api_lib.iam import util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as gcloud_exceptions
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.iam import identity_pool_waiter
from googlecloudsdk.command_lib.util.apis import yaml_data
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import log
from googlecloudsdk.core import resources as sdkresources
from googlecloudsdk.core import yaml


@base.Hidden
@base.UniverseCompatible
class AddAttestationRule(base.Command):
  """Set attestation rules on a workload identity pool namespace."""

  detailed_help = {
      'DESCRIPTION': '{description}',
      'EXAMPLES': """\
          The following command sets attestation rules on a workload identity
          pool namespace `my-namespace` using a policy file.

            $ {command} my-namespace \
            --workload-identity-pool="my-workload-identity-pool" \
            --location="global" \
            --policy-file="policy.json"
          """,
  }

  @staticmethod
  def Args(parser):
    namespace_data = yaml_data.ResourceYAMLData.FromPath(
        'iam.workload_identity_pool_namespace'
    )
    concept_parsers.ConceptParser.ForResource(
        'namespace',
        concepts.ResourceSpec.FromYaml(
            namespace_data.GetData(), is_positional=True
        ),
        'The workload identity pool namespace to set attestations rule on.',
        required=True,
    ).AddToParser(parser)
    parser.add_argument(
        '--policy-file',
        help="""\
          Path to a local JSON-formatted or YAML-formatted file containing an
          attestation policy, structured as a [list of attestation rules](https://cloud.google.com/iam/docs/reference/rest/v1/projects.locations.workloadIdentityPools.namespaces/setAttestationRules#request-body).
          """,
        required=True,
    )
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    client, messages = util.GetClientAndMessages()
    namespace_ref = args.CONCEPTS.namespace.Parse()

    policy_to_parse = yaml.load_path(args.policy_file)

    try:
      set_attestation_rules_request = encoding.PyValueToMessage(
          messages.SetAttestationRulesRequest, policy_to_parse
      )
    except AttributeError as e:
      # Raised when the input file is not properly formatted YAML policy file.
      raise gcloud_exceptions.BadFileException(
          'Policy file [{0}] is not a properly formatted YAML or JSON '
          'policy file. {1}'.format(args.policy_file, str(e))
      )

    lro_ref = client.projects_locations_workloadIdentityPools_namespaces.SetAttestationRules(
        messages.IamProjectsLocationsWorkloadIdentityPoolsNamespacesSetAttestationRulesRequest(
            resource=namespace_ref.RelativeName(),
            setAttestationRulesRequest=set_attestation_rules_request,
        )
    )

    log.status.Print(
        'Set attestation rules request issued for: [{}]'.format(
            namespace_ref.namespacesId
        )
    )

    if args.async_:
      return lro_ref

    result = waiter.WaitFor(
        poller=identity_pool_waiter.IdentityPoolOperationPollerNoResources(
            client.projects_locations_workloadIdentityPools_namespaces,
            client.projects_locations_workloadIdentityPools_namespaces_operations,
        ),
        operation_ref=sdkresources.REGISTRY.ParseRelativeName(
            lro_ref.name,
            collection=(
                'iam.projects.locations.workloadIdentityPools.namespaces.operations'
            ),
        ),
        message='Waiting for operation [{}] to complete'.format(lro_ref.name),
        # Wait for a maximum of 5 minutes, as the IAM replication has a lag of
        # up to 80 seconds.
        max_wait_ms=300000,
    )

    log.status.Print(
        'Set attestation rules for [{}].'.format(namespace_ref.namespacesId)
    )

    return result



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to update a cluster-director cluster resource."""

import textwrap

from googlecloudsdk.api_lib.hypercomputecluster import utils as api_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.cluster_director.clusters import flags
from googlecloudsdk.command_lib.cluster_director.clusters import utils
from googlecloudsdk.core import log


DETAILED_HELP = {
    "DESCRIPTION": textwrap.dedent("""
        *{command}* facilitates the updation of a cluster resource.

        There are following ways to update a cluster:
        - [Preferred] Use granular flags to update cluster specs, based on read-modify-update pattern.
          - Read the existing cluster specs through `get` cluster request.
          - Modify the cluster specs through granular flags.
          - Update the cluster specs through `patch` cluster request.
        - Use --config with cluster specs and --update-mask flags, both in JSON format.
          - Map and repeated fields update requires existing and new values.
          - For e.g. if we want to update a cluster to add a new nodeset, then we will use the update_mask "orchestrator.slurm.node_sets", and the patch cluster must include all the existing nodesets as well as the new one.

        Please refer to the examples below for more details.
        """),
    "EXAMPLES": textwrap.dedent("""
        To update a cluster `my-cluster` in location `us-central1-a` with granular flags, run the following example:

        Add labels, compute instances, slurm node sets, slurm partitions and update description and default partition:

        $ {command} my-cluster --location us-central1-a \
        --description "My updated cluster description" \
        --add-labels env=prod,client=gcloud-cli \
        --add-on-demand-instances id=compute1,zone=us-central1-a,machineType=n2-standard-2 \
        --add-reserved-instances id=compute2,reservation=zones/us-central1-a/reservations/{reservation} \
        --add-spot-instances id=compute3,zone=us-central1-a,machineType=n2-standard-2 \
        --add-dws-flex-instances id=compute4,zone=us-central1-a,machineType=a4-highgpu-8g,maxDuration=10000s \
        --add-new-lustre-instances lustre=locations/us-central1-a/instances/lustre1,capacityGb=1024,filesystem=lustre1 \
        --add-lustre-instances locations/us-central1-a/instances/lustre0 \
        --add-slurm-node-sets id=nodeset1,computeId=compute1 \
        --add-slurm-node-sets id=nodeset2,computeId=compute2 \
        --add-slurm-node-sets id=nodeset3,computeId=compute3 \
        --add-slurm-node-sets id=nodeset4,computeId=compute4 \
        --add-slurm-partitions id=partition1,nodesetIds=[nodeset1] \
        --add-slurm-partitions id=partition2,nodesetIds=[nodeset2,nodeset3,nodeset4] \
        --slurm-default-partition partition1

        Update slurm node sets and slurm partitions:

        $ {command} my-cluster --location us-central1-a \
        --update-slurm-node-sets id=nodeset1,staticNodeCount=2,maxDynamicNodeCount=10 \
        --update-slurm-partitions id=partition1,nodesetIds=[nodeset0],exclusive=true

        Remove slurm node sets, slurm partitions and compute instances and update default partition:

        $ {command} my-cluster --location us-central1-a \
        --slurm-default-partition partition0 \
        --remove-labels env,client \
        --remove-lustre-instances locations/us-central1-a/instances/lustre0 \
        --remove-slurm-partitions partition1 \
        --remove-slurm-partitions partition2 \
        --remove-slurm-node-sets nodeset1 \
        --remove-slurm-node-sets nodeset2 \
        --remove-slurm-node-sets nodeset3 \
        --remove-slurm-node-sets nodeset4 \
        --remove-on-demand-instances compute1 \
        --remove-reserved-instances compute2 \
        --remove-spot-instances compute3 \
        --remove-dws-flex-instances compute4

        Add and remove slurm prolog/epilog scripts:

        $ {command} my-cluster --location us-central1-a \
        --add-slurm-prolog-scripts script1.sh \
        --remove-slurm-epilog-scripts script2.sh

        Or cluster `my-cluster` in location `us-central1-a` with config JSON run the following JSON example:

        $ {command} my-cluster --location=us-central1-a --update-mask=labels --config='{"key": "value"}'

        Or create a JSON file `my-cluster-config.json` with the cluster specs and run the following file example:

        $ {command} my-cluster --location=us-central1-a --update-mask=labels --config-from-file=my-cluster-config.json

        Or create a JSON file with the update mask and run the following file example:

        $ {command} my-cluster --location=us-central1-a --update-mask-from-file=my-update-mask.json --config-from-file=my-cluster-config.json
        """),
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Update(base.UpdateCommand):
  """Updates a Cluster Director resource."""

  @classmethod
  def Args(cls, parser):
    """Specifies command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    base.ASYNC_FLAG.AddToParser(parser)
    api_version = api_utils.GetApiVersion(cls.ReleaseTrack())
    utils.AddClusterNameArgToParser(parser=parser, api_version=api_version)
    group = parser.add_group(
        help="Cluster configuration for provisioning with updates.",
        mutex=True,
        required=True,
    )
    config_group = group.add_group(
        help="Cluster configuration for updates.",
    )
    flags.AddConfig(parser=config_group, api_version=api_version, required=True)
    flags.AddUpdateMask(
        parser=config_group, api_version=api_version, required=True
    )
    flag_group = group.add_group(
        help="Flag Configurations to define cluster updates.",
    )
    flags.AddDescription(parser=flag_group, api_version=api_version)
    flags.AddLabels(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddCreateFilestores(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddFilestores(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddGcsBuckets(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddCreateGcsBuckets(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddCreateLustres(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddLustres(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddOnDemandInstances(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddSpotInstances(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddReservedInstances(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddFlexStartInstances(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddSlurmNodeSets(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddSlurmPartitions(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddSlurmDefaultPartition(parser=flag_group, api_version=api_version)
    flags.AddSlurmLoginNode(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddSlurmPrologBashScripts(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    flags.AddSlurmEpilogBashScripts(
        parser=flag_group, api_version=api_version, include_update_flags=True
    )
    if api_version == "v1alpha":
      flags.AddSlurmTaskPrologBashScripts(
          parser=flag_group, api_version=api_version, include_update_flags=True
      )
      flags.AddSlurmTaskEpilogBashScripts(
          parser=flag_group, api_version=api_version, include_update_flags=True
      )
      flags.AddSlurmConfig(
          parser=flag_group, api_version=api_version, include_update_flags=True
      )
      flags.AddSlurmDisableHealthCheckProgram(
          parser=flag_group, api_version=api_version, include_update_flags=True
      )

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    release_track = self.ReleaseTrack()
    client = api_utils.GetClientInstance(release_track)
    messages = api_utils.GetMessagesModule(release_track)
    existing_cluster = None
    if not args.IsSpecified("config"):
      existing_cluster = api_utils.GetCluster(client, args, messages)
    cluster_util = utils.ClusterUtil(
        args=args, message_module=messages, existing_cluster=existing_cluster
    )
    cluster_ref = cluster_util.cluster_ref

    if args.IsSpecified("config"):
      cluster_patch, update_mask = cluster_util.MakeClusterPatchFromConfig()
    else:
      cluster_patch, update_mask = cluster_util.MakeClusterPatch()

    operation = client.projects_locations_clusters.Patch(
        messages.HypercomputeclusterProjectsLocationsClustersPatchRequest(
            name=cluster_ref.RelativeName(),
            cluster=cluster_patch,
            updateMask=update_mask,
        )
    )
    log.status.Print(
        "Update request issued for: [{0}]".format(cluster_ref.Name())
    )
    if args.async_:
      return operation

    response = api_utils.WaitForOperation(
        client=client,
        operation=operation,
        message="Waiting for operation [{0}] to complete".format(
            operation.name
        ),
        max_wait_sec=3600,
    )
    log.status.Print("Updated cluster [{0}].".format(cluster_ref.Name()))
    return response


Update.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from googlecloudsdk.generated_clients.gapic_clients.run_v2.types import k8s_min


__protobuf__ = proto.module(
    package='google.cloud.run.v2',
    manifest={
        'SandboxConfiguration',
    },
)


class SandboxConfiguration(proto.Message):
    r"""Configuration for sandboxes.

    Attributes:
        templates (MutableSequence[googlecloudsdk.generated_clients.gapic_clients.run_v2.types.Container]):
            Required. Container templates that can be launched through
            the ``sandbox`` CLI.
    """

    templates: MutableSequence[k8s_min.Container] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=k8s_min.Container,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
